@@ -15,11 +15,12 @@
 #
 
 import contextlib
+import itertools
 import sys
-import urllib
 
 import fixtures
 from mox3 import mox
+from oslo.utils import encodeutils
 from oslotest import base
 import requests
 import six
@@ -27,6 +28,7 @@ import six.moves.urllib.parse as urlparse
 
 from neutronclient.common import constants
 from neutronclient.common import exceptions
+from neutronclient.common import utils
 from neutronclient.neutron import v2_0 as neutronV2_0
 from neutronclient import shell
 from neutronclient.v2_0 import client
@@ -175,17 +177,13 @@ class CLITestV20Base(base.BaseTestCase):
 
     def _get_attr_metadata(self):
         return self.metadata
-        client.Client.EXTED_PLURALS.update(constants.PLURALS)
-        client.Client.EXTED_PLURALS.update({'tags': 'tag'})
-        return {'plurals': client.Client.EXTED_PLURALS,
-                'xmlns': constants.XML_NS_V20,
-                constants.EXT_NS: {'prefix': 'http://xxxx.yy.com'}}
 
-    def setUp(self, plurals={}):
+    def setUp(self, plurals=None):
         """Prepare the test environment."""
         super(CLITestV20Base, self).setUp()
         client.Client.EXTED_PLURALS.update(constants.PLURALS)
-        client.Client.EXTED_PLURALS.update(plurals)
+        if plurals is not None:
+            client.Client.EXTED_PLURALS.update(plurals)
         self.metadata = {'plurals': client.Client.EXTED_PLURALS,
                          'xmlns': constants.XML_NS_V20,
                          constants.EXT_NS: {'prefix':
@@ -219,7 +217,8 @@ class CLITestV20Base(base.BaseTestCase):
                                       'credential', 'network_profile',
                                       'policy_profile', 'ikepolicy',
                                       'ipsecpolicy', 'metering_label',
-                                      'metering_label_rule', 'net_partition']
+                                      'metering_label_rule', 'net_partition',
+                                      'fox_socket']
         if not cmd_resource:
             cmd_resource = resource
         if (resource in non_admin_status_resources):
@@ -271,7 +270,7 @@ class CLITestV20Base(base.BaseTestCase):
             self.assertIn(name, _str)
 
     def _test_list_columns(self, cmd, resources,
-                           resources_out, args=['-f', 'json'],
+                           resources_out, args=('-f', 'json'),
                            cmd_resources=None, parent_id=None):
         self.mox.StubOutWithMock(cmd, "get_client")
         self.mox.StubOutWithMock(self.client.httpclient, "request")
@@ -290,18 +289,18 @@ class CLITestV20Base(base.BaseTestCase):
             body=None,
             headers=mox.ContainsKeyValue(
                 'X-Auth-Token', TOKEN)).AndReturn((MyResp(200), resstr))
-        args.extend(['--request-format', self.format])
+        args = tuple(args) + ('--request-format', self.format)
         self.mox.ReplayAll()
         cmd_parser = cmd.get_parser("list_" + cmd_resources)
         shell.run_command(cmd, cmd_parser, args)
         self.mox.VerifyAll()
         self.mox.UnsetStubs()
 
-    def _test_list_resources(self, resources, cmd, detail=False, tags=[],
-                             fields_1=[], fields_2=[], page_size=None,
-                             sort_key=[], sort_dir=[], response_contents=None,
+    def _test_list_resources(self, resources, cmd, detail=False, tags=(),
+                             fields_1=(), fields_2=(), page_size=None,
+                             sort_key=(), sort_dir=(), response_contents=None,
                              base_args=None, path=None, cmd_resources=None,
-                             parent_id=None):
+                             parent_id=None, output_format=None):
         self.mox.StubOutWithMock(cmd, "get_client")
         self.mox.StubOutWithMock(self.client.httpclient, "request")
         cmd.get_client().MultipleTimes().AndReturn(self.client)
@@ -331,12 +330,12 @@ class CLITestV20Base(base.BaseTestCase):
             args.append("--tag")
         for tag in tags:
             args.append(tag)
-            if isinstance(tag, unicode):
-                tag = urllib.quote(tag.encode('utf-8'))
+            tag_query = urlparse.urlencode(
+                {'tag': encodeutils.safe_encode(tag)})
             if query:
-                query += "&tag=" + tag
+                query += "&" + tag_query
             else:
-                query = "tag=" + tag
+                query = tag_query
         if (not tags) and fields_2:
             args.append('--')
         if fields_2:
@@ -345,8 +344,7 @@ class CLITestV20Base(base.BaseTestCase):
                 args.append(field)
         if detail:
             query = query and query + '&verbose=True' or 'verbose=True'
-        fields_1.extend(fields_2)
-        for field in fields_1:
+        for field in itertools.chain(fields_1, fields_2):
             if query:
                 query += "&fields=" + field
             else:
@@ -368,7 +366,7 @@ class CLITestV20Base(base.BaseTestCase):
         if sort_dir:
             len_diff = len(sort_key) - len(sort_dir)
             if len_diff > 0:
-                sort_dir += ['asc'] * len_diff
+                sort_dir = tuple(sort_dir) + ('asc',) * len_diff
             elif len_diff < 0:
                 sort_dir = sort_dir[:len(sort_key)]
             for dir in sort_dir:
@@ -381,6 +379,9 @@ class CLITestV20Base(base.BaseTestCase):
             path = getattr(self.client, cmd_resources + "_path")
             if parent_id:
                 path = path % parent_id
+        if output_format:
+            args.append('-f')
+            args.append(output_format)
         self.client.httpclient.request(
             MyUrlComparator(end_url(path, query, format=self.format),
                             self.client),
@@ -399,6 +400,7 @@ class CLITestV20Base(base.BaseTestCase):
         return _str
 
     def _test_list_resources_with_pagination(self, resources, cmd,
+                                             base_args=None,
                                              cmd_resources=None,
                                              parent_id=None):
         self.mox.StubOutWithMock(cmd, "get_client")
@@ -433,7 +435,8 @@ class CLITestV20Base(base.BaseTestCase):
                 'X-Auth-Token', TOKEN)).AndReturn((MyResp(200), resstr2))
         self.mox.ReplayAll()
         cmd_parser = cmd.get_parser("list_" + cmd_resources)
-        args = ['--request-format', self.format]
+        args = base_args if base_args is not None else []
+        args.extend(['--request-format', self.format])
         shell.run_command(cmd, cmd_parser, args)
         self.mox.VerifyAll()
         self.mox.UnsetStubs()
@@ -475,7 +478,7 @@ class CLITestV20Base(base.BaseTestCase):
         _str = self.fake_stdout.make_string()
         self.assertIn(myid, _str)
 
-    def _test_show_resource(self, resource, cmd, myid, args, fields=[],
+    def _test_show_resource(self, resource, cmd, myid, args, fields=(),
                             cmd_resource=None, parent_id=None):
         self.mox.StubOutWithMock(cmd, "get_client")
         self.mox.StubOutWithMock(self.client.httpclient, "request")
@@ -566,17 +569,16 @@ class ClientV2TestJson(CLITestV20Base):
         unicode_text = u'\u7f51\u7edc'
         # url with unicode
         action = u'/test'
-        expected_action = action.encode('utf-8')
+        expected_action = action
         # query string with unicode
         params = {'test': unicode_text}
-        expect_query = urllib.urlencode({'test':
-                                         unicode_text.encode('utf-8')})
+        expect_query = urlparse.urlencode(utils.safe_encode_dict(params))
         # request body with unicode
         body = params
         expect_body = self.client.serialize(body)
-        # headers with unicode
-        self.client.httpclient.auth_token = unicode_text
-        expected_auth_token = unicode_text.encode('utf-8')
+        self.client.httpclient.auth_token = encodeutils.safe_encode(
+            unicode_text)
+        expected_auth_token = encodeutils.safe_encode(unicode_text)
 
         self.client.httpclient.request(
             end_url(expected_action, query=expect_query, format=self.format),
@@ -598,7 +600,7 @@ class ClientV2TestJson(CLITestV20Base):
         self.client.format = self.format
         self.mox.StubOutWithMock(self.client.httpclient, "request")
         params = {'test': 'value'}
-        expect_query = urllib.urlencode(params)
+        expect_query = six.moves.urllib.parse.urlencode(params)
         self.client.httpclient.auth_token = 'token'
 
         self.client.httpclient.request(
@@ -665,6 +667,7 @@ class CLITestV20ExceptionHandler(CLITestV20Base):
             ('ExternalIpAddressExhausted',
              exceptions.ExternalIpAddressExhaustedClient, 400),
             ('OverQuota', exceptions.OverQuotaClient, 409),
+            ('InvalidIpForNetwork', exceptions.InvalidIpForNetworkClient, 400),
         ]
 
         error_msg = 'dummy exception message'
